# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "alifax"


def test_query(self):
    result = self.decode("Q0111212726601    0000")
    expected = {
        "context": "query",
        "samples": [{"sample_id": "11212726601", "type": "patient"}],
    }
    assert result == expected


def test_void_query(self):
    result = self.order(
        worklist=self.void_worklist,
        context={"query": self.parse("Q0111212726601    0000")},
    )
    expected = "T  11212726601    0000"
    assert result == expected


def test_non_result(self):
    result = self.decode("T01123456         0001")
    expected = {"context": "result", "samples": []}
    assert result == expected


def test_result(self):
    result = self.decode("R012320309601     0604010032")
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "2320309601",
                "results": [{"code": "VS1", "value": "0032"}],
                "type": "patient",
            }
        ],
    }
    assert result == expected


def test_qc(self):
    date = datetime.datetime.now()
    date = date.replace(second=0, microsecond=0)
    result = self.decode("R01LATEX03663065  0103010067")
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "LATEX03663",
                "results": [{"code": "VS1", "value": "0067", "date": date}],
                "sample_id": "LATEX03663",
                "type": "control",
            }
        ],
    }
    assert result == expected


def test_order(self):
    result = self.order()
    expected = "T01123456         0001"
    assert result == expected
