# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

import driverng

parser_class = "atellica_astm"


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    # test d'un code medecin trop long
    worklist.samples[0].physicians[0].code += "CODE_MEDCIN_TROP_LONG"
    message = self.order(worklist=worklist)
    expected = """
    H|\\^&||||||||||P|1
    P|1|036666666|L036666666||NYME^ANO||20110513|F|||||ATTENDINGCOD^LAGRANGE||||||||||||LOC
    O|1|123456||^^^A^^1:10|S|20130603110342|20130513114242||||N||||Serum||||||||||O
    O|2|123456||^^^B|S|20130603110342|20130513114242||||N||||Serum||||||||||O
    O|3|123456||^^^C|S|20130603110342|20130513114242||||C||||Serum||||||||||O
    L|1|F
    """
    assert message == expected


def test_encode_enable_disable(self):
    worklist = driverng.Worklist(
        context="order",
        samples=[
            {
                "sample_id": "INSTRUMENT:ADV1800-1",
                "type": "patient",
                "specimen": "Urine",
                "orders": [
                    {"code": "CK", "action": "add"},
                    {"code": "NA", "action": "del"},
                ],
            },
            {
                "sample_id": "INSTRUMENT:ADV1800-2",
                "type": "patient",
                "specimen": "Serum",
                "orders": [{"code": "*", "action": "del"}],
            },
        ],
    )
    message = self.encode(worklist)
    # print ( self.parser.to_string(message))
    assert (
        message
        == """
    H|\\^&||||||||||P|1
    M|1|SHD^CEN:NG^V1^S|CK|Urine|ENABLE|ADV1800^1
    M|2|SHD^CEN:NG^V1^S|NA|Urine|DISABLE|ADV1800^1
    M|3|SHD^CEN:NG^V1^S|*|Serum|DISABLE|ADV1800^2
    L|1|F"""
    )


def test_decode_result(self):
    self.configure(atellica_separator=".")
    date = datetime.datetime(2016, 10, 19, 13, 13, 1)
    worklist = self.decode(
        """
    H|\\^&|||UIW_LIS|||||LIS_ID||P||20161019131303
    P|1|1057|LD00001||Beck^Alexandrea^j||20050411|F|||||PH000003|Human||||||||||Critical|ER
    O|1|Sample20001||^^^CA125^^^3|R|20161019131046|20161018120914|||||||20161018130957|Serum^^^1||||||||||F
    C|1||^Sample is good|G
    R|1|^^^CA125^^^3^RLU^171#0|57395|||||F||SiemensInternal^No Review||20161019131301|5-5-5-5-5-5-5^1-1-1-1-1-1-1|1|I|Operator Interpretation OP|I
    C|2||PREAG^103|G
    R|2|^^^CA125^^^3^COFF^171#0|1.0|U/mL||||F||SiemensInternal^NoReview||20161019131301|5-5-5-5-5-5-5^1-1-1-1-1-1-1
    C|1|I|Operator Interpretation OP|I
    C|2||PREAG^103|G
    R|3|^^^CA125^^^3^DOSE^171#0|25.91|U/mL||||F||SiemensInternal^NoReview||20161019131301|5-5-5-5-5-5-5^1-1-1-1-1-1-1
    C|1|I|Operator Interpretation OP|I
    C|2||PREAG^103|G
    R|4|^^^CA125^^^3^TEST^171#0|25.92|U/mL||||P||SiemensInternal^NoReview||20161019131301|5-5-5-5-5-5-5^1-1-1-1-1-1-1
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "Sample20001",
                "rack": "",
                "specimen": "Serum",
                "results": [
                    {
                        "code": "CA125.RLU",
                        "date": date,
                        "flags": [],
                        "comments": [],
                        "instrument": "1-1-1-1-1-1-1",
                        "value": "57395",
                        "unit": "",
                        "operator": "SiemensInternal",
                        "dilution": "",
                        "lots": [{"type": "reagent", "lot": "103"}],
                    },
                    {
                        "code": "CA125.COFF",
                        "date": date,
                        "flags": ["Operator Interpretation OP"],
                        "comments": [],
                        "instrument": "1-1-1-1-1-1-1",
                        "value": "1.0",
                        "unit": "U/mL",
                        "operator": "SiemensInternal",
                        "dilution": "",
                        "lots": [{"type": "reagent", "lot": "103"}],
                    },
                    {
                        "code": "CA125.DOSE",
                        "date": date,
                        "flags": ["Operator Interpretation OP"],
                        "comments": [],
                        "instrument": "1-1-1-1-1-1-1",
                        "value": "25.91",
                        "unit": "U/mL",
                        "operator": "SiemensInternal",
                        "dilution": "",
                        "lots": [{"type": "reagent", "lot": "103"}],
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    self.configure(atellica_separator="")
    date = datetime.datetime(2016, 10, 19, 14, 30, 39)
    worklist = self.decode(
        """
    H|\\^&|||UIW_LIS|||||LIS_ID||P||20161019143043
    P|1
    O|1|Q1||^^^CA125^^^1|R|20161019142951|||||Q||||Serum||||||||||F
    M|1|SHD^CEN:NG^V1^O|QOne|4444
    R|1|^^^CA125^^^1^RLU^172#0|2803|||L||F||SiemensInternal^NoReview||20161019143039|5-5-5-5-5-5-5^1-1-1-1-1-1-1
    C|1|I|Low|I
    C|2||PREAG^103|G
    C|3||CALINFO^Lot=C015;DateTime=20161013101022;Status=Due;|G
    R|2|^^^CA125^^^1^COFF^172#0|1.0|U/mL||L||F||SiemensInternal^NoReview||20161019143039|5-5-5-5-5-5-5^1-1-1-1-1-1-1
    C|1|I|Low|I
    C|2||PREAG^103|G
    C|3||CALINFO^Lot=C015;DateTime=20161013101022;Status=Due;|G
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "Q1",
                "control_lot": "4444",
                "control_name": "QOne",
                "specimen": "Serum",
                "results": [
                    {
                        "code": "CA125RLU",
                        "date": date,
                        "flags": [],
                        "comments": [],
                        "instrument": "1-1-1-1-1-1-1",
                        "value": "2803",
                        "unit": "",
                        "operator": "SiemensInternal",
                        "dilution": "",
                        "lots": [{"type": "reagent", "lot": "103"}],
                    },
                    {
                        "code": "CA125COFF",
                        "date": date,
                        "flags": [],
                        "comments": [],
                        "instrument": "1-1-1-1-1-1-1",
                        "value": "1.0",
                        "unit": "U/mL",
                        "operator": "SiemensInternal",
                        "dilution": "",
                        "lots": [{"type": "reagent", "lot": "103"}],
                    },
                ],
            }
        ],
    }
    assert worklist == expected
