# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "autoplak"


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    message = self.order(
        worklist=worklist,
        context={"control_id": "1521651964637", "placer_order_number": "1234547"},
    )
    expected = """
    MSH|^~\\&|LIS01|Microbiology|AUTOPLAK01|MicroLab|20130715141912||RSP^K11^RSP_K11|MSG1521651964637|P|2.5
    MSA|AA|MSG1521651964637
    QAK|MSG00001|OK
    QPD|WOS^Work Order Step|MSG1521651964637|123456
    SPM|1|123456||Serum^Serum|||||||P||||||20130513114242
    PID|1||||ANO^NYME||20110513|F
    ORC|NW|1234547|||||||20130603110342
    TQ1|1||||||||S
    OBR|1|1234547||A
    ORC|NW|1234547|||||||20130603110342
    TQ1|1||||||||S
    OBR|1|1234547||B
    """
    assert message == expected


def test_encode_void_order(self):
    message = self.order(
        worklist=self.void_worklist,
        context={
            "control_id": "1521651964637",
            "query": self.parse(
                """
            MSH|^~\\&|AUTOPLAK01|MicroLab|LIS01|Microbiology|20140314134213||QBP^Q11^QBP_Q11|MSG20140314134213|P|2.5
            QPD|WOS^Work Order Step|MSG20140314134213|1234567890
            RCP|I||R
            """
            ),
        },
    )
    expected = """
    MSH|^~\\&|LIS01|Microbiology|AUTOPLAK01|MicroLab|20130715141912||RSP^K11^RSP_K11|MSG1521651964637|P|2.5
    MSA|AA|MSG1521651964637
    QAK|MSG00001|NF
    QPD|WOS^Work Order Step|MSG1521651964637|1234567890
    """
    assert message == expected


def test_decode_result(self):
    date = datetime.datetime(2013, 3, 5, 9, 25, 44)
    worklist = self.decode(
        """
    MSH|^~\\&|AUTOPLAK|Microbiology|LIS01||20130305092544||OUL^R22^OUL_R22|MSG00001|P|2.5
    SPM|1|A239852|1234567890|UR^URINE|||||||P|||||20130304223102
    SAC|||1234567890^AUTOPLAK|||||R^COMPLETED|RACK||7|DRAWER|0|4||||||||||||||||USR1
    OBR|1|98765_1||URIGLEB|||||||||||||||||||||F
    ORC|SC|98765_1||98765|||||20130112131517
    OBX|1||URIGLEB||||||||F
    OBR|1|98765_2||URIGRAM|||||||||||||||||||||F
    ORC|SC|98765_2||98765|||||20130112131517
    OBX|1||URIGRAM||||||||F
    OBR|1|98765_3||URIESP|||||||||||||||||||||F
    ORC|SC|98765_3||98765|||||20130112131517
    OBX|1||URIESP||||||||X
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "URDATETIME",
                        "value": "2013/01/12 13:15:17",
                        "date": date,
                        "operator": "USR1",
                        "instrument": "AUTOPLAK",
                    },
                    {
                        "code": "URID",
                        "value": "1234567890",
                        "date": date,
                        "operator": "USR1",
                        "instrument": "AUTOPLAK",
                    },
                    {
                        "code": "URSTATUS",
                        "value": "F",
                        "date": date,
                        "operator": "USR1",
                        "instrument": "AUTOPLAK",
                    },
                    {
                        "code": "URDATETIME",
                        "value": "2013/01/12 13:15:17",
                        "date": date,
                        "operator": "USR1",
                        "instrument": "AUTOPLAK",
                    },
                    {
                        "code": "URID",
                        "value": "1234567890",
                        "date": date,
                        "operator": "USR1",
                        "instrument": "AUTOPLAK",
                    },
                    {
                        "code": "URSTATUS",
                        "value": "F",
                        "date": date,
                        "operator": "USR1",
                        "instrument": "AUTOPLAK",
                    },
                    {
                        "code": "URDATETIME",
                        "value": "2013/01/12 13:15:17",
                        "date": date,
                        "operator": "USR1",
                        "instrument": "AUTOPLAK",
                    },
                    {
                        "code": "URID",
                        "value": "1234567890",
                        "date": date,
                        "operator": "USR1",
                        "instrument": "AUTOPLAK",
                    },
                    {
                        "code": "URSTATUS",
                        "value": "X",
                        "date": date,
                        "operator": "USR1",
                        "instrument": "AUTOPLAK",
                    },
                    {
                        "code": "UR_BOITES",
                        "value": "URIGLEB;URIGRAM;URIESP",
                        "date": date,
                        "operator": "USR1",
                        "instrument": "AUTOPLAK",
                    },
                ],
                "type": "patient",
                "rack": "RACK-7_DRAWER-0-4",
                "sample_id": "A239852",
            }
        ],
    }
    assert worklist == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    MSH|^~\\&|AUTOPLAK01|MicroLab|LIS01|Microbiology|20140314134213||QBP^Q11^QBP_Q11|MSG2014031134213|P|2.5
    QPD|WOS^Work Order Step|MSG20140314134213|1234567890
    RCP|I||R
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "1234567890"}],
    }
    assert worklist == expected
