# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "clinitek_advantus"


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&|||SIEMENS^1420^0310628^03.10:030/V3.10||||||010|P|1|20101207152200
    P|1|00001|20070526102500|312445446|323
    P|2|C|P|MULTISTIX 10 SG|F|0000|7E03P|2014-11|
    R|1|N|COL|12|YELLOW|1|0|A
    R|2|A|GLU|1|>=1000^mg/dL|3|0|A||High SG may cause falsely lowered GLU results
    R|3|N|BIL|2|NEGATIVE|1|0|A||
    R|4|N|KET|3|NEGATIVE|1|0|E||
    R|5|N|SG|4|<=1.005|1|0|A||
    R|6|A|BLO|9|SMALL|4|0|A||
    R|7|N|pH|5|6.0|3|0|A||
    R|8|N|PRO|6|NEGATIVE|1|0|A||
    R|9|N|URO|7|1.0^E.U./dL|2|0|A||
    R|10|N|NIT|8|NEGATIVE|1|0|A||
    R|11|A|LEU|10|SMALL|3|0|A||Elevated GLU may cause falsely lowered LEU results^High SG may cause falsely lowered LEU results
    L|1
    """
    )
    date = datetime.datetime(2007, 5, 26, 10, 25)
    reagent_expiry = datetime.datetime(2014, 11, 1, 0, 0)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "COL",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            }
                        ],
                        "unit": "",
                        "value": "YELLOW",
                    },
                    {
                        "code": "GLU",
                        "comments": [
                            "High SG may cause falsely lowered GLU " "results"
                        ],
                        "date": date,
                        "flags": ["3"],
                        "lots": [
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            }
                        ],
                        "unit": "mg/dL",
                        "value": ">=1000",
                    },
                    {
                        "code": "BIL",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            }
                        ],
                        "unit": "",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "KET",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            }
                        ],
                        "unit": "",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "SG",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            }
                        ],
                        "unit": "",
                        "value": "<=1.005",
                    },
                    {
                        "code": "BLO",
                        "comments": [],
                        "date": date,
                        "flags": ["4"],
                        "lots": [
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            }
                        ],
                        "unit": "",
                        "value": "SMALL",
                    },
                    {
                        "code": "pH",
                        "comments": [],
                        "date": date,
                        "flags": ["3"],
                        "lots": [
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            }
                        ],
                        "unit": "",
                        "value": "6.0",
                    },
                    {
                        "code": "PRO",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            }
                        ],
                        "unit": "",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "URO",
                        "comments": [],
                        "date": date,
                        "flags": ["2"],
                        "lots": [
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            }
                        ],
                        "unit": "E.U./dL",
                        "value": "1.0",
                    },
                    {
                        "code": "NIT",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            }
                        ],
                        "unit": "",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "LEU",
                        "comments": [
                            "Elevated GLU may cause falsely " "lowered LEU results",
                            "High SG may cause falsely lowered LEU " "results",
                        ],
                        "date": date,
                        "flags": ["3"],
                        "lots": [
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            }
                        ],
                        "unit": "",
                        "value": "SMALL",
                    },
                ],
                "sample_id": "312445446",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    H|\\^&|||SIEMENS^1420^0310628^03.10:030/V3.10||||||010|Q|1|20101207152200
    P|1|C0002|20070526102500|213543331|323
    P|2|C|C|MULTISTIX 10 SG|F|0000|7E03P|2014-11|2014-12|
    R|1|N|COL|12|YELLOW|1|0|A
    R|2|N|GLU|1|NEGATIVE|1|0|A||
    R|3|N|BIL|2|NEGATIVE|1|0|A||
    R|4|N|KET|3|NEGATIVE|1|0|E||
    R|5|N|SG|4|<=1.005|1|0|A||
    R|6|N|BLO|9|NEGATIVE|1|0|A||
    R|7|N|pH|5|5.5|2|0|A||
    R|8|N|PRO|6|NEGATIVE|1|0|A||Visibly bloody urine may cause falsely elevated PRO results^High pH may cause falsely elevated PRO results
    R|9|N|URO|7|0.2^E.U./dL|1|0|A||
    R|10|N|NIT|8|NEGATIVE|1|0|A||
    R|11|N|LEU|10|NEGATIVE|1|0|A||
    L|1
    """
    )
    date = datetime.datetime(2007, 5, 26, 10, 25)
    cq_expiry = datetime.datetime(2014, 12, 1, 0, 0)
    reagent_expiry = datetime.datetime(2014, 11, 1, 0, 0)
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "213543331",
                "results": [
                    {
                        "code": "COL",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": cq_expiry,
                                "lot": "213543331",
                                "type": "control",
                            },
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            },
                        ],
                        "unit": "",
                        "value": "YELLOW",
                    },
                    {
                        "code": "GLU",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": cq_expiry,
                                "lot": "213543331",
                                "type": "control",
                            },
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            },
                        ],
                        "unit": "",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "BIL",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": cq_expiry,
                                "lot": "213543331",
                                "type": "control",
                            },
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            },
                        ],
                        "unit": "",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "KET",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": cq_expiry,
                                "lot": "213543331",
                                "type": "control",
                            },
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            },
                        ],
                        "unit": "",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "SG",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": cq_expiry,
                                "lot": "213543331",
                                "type": "control",
                            },
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            },
                        ],
                        "unit": "",
                        "value": "<=1.005",
                    },
                    {
                        "code": "BLO",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": cq_expiry,
                                "lot": "213543331",
                                "type": "control",
                            },
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            },
                        ],
                        "unit": "",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "pH",
                        "comments": [],
                        "date": date,
                        "flags": ["2"],
                        "lots": [
                            {
                                "expiration": cq_expiry,
                                "lot": "213543331",
                                "type": "control",
                            },
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            },
                        ],
                        "unit": "",
                        "value": "5.5",
                    },
                    {
                        "code": "PRO",
                        "comments": [
                            "Visibly bloody urine may cause "
                            "falsely elevated PRO results",
                            "High pH may cause falsely elevated " "PRO results",
                        ],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": cq_expiry,
                                "lot": "213543331",
                                "type": "control",
                            },
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            },
                        ],
                        "unit": "",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "URO",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": cq_expiry,
                                "lot": "213543331",
                                "type": "control",
                            },
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            },
                        ],
                        "unit": "E.U./dL",
                        "value": "0.2",
                    },
                    {
                        "code": "NIT",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": cq_expiry,
                                "lot": "213543331",
                                "type": "control",
                            },
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            },
                        ],
                        "unit": "",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "LEU",
                        "comments": [],
                        "date": date,
                        "flags": ["1"],
                        "lots": [
                            {
                                "expiration": cq_expiry,
                                "lot": "213543331",
                                "type": "control",
                            },
                            {
                                "expiration": reagent_expiry,
                                "lot": "7E03P",
                                "name": "MULTISTIX 10 SG",
                                "type": "reagent",
                            },
                        ],
                        "unit": "",
                        "value": "NEGATIVE",
                    },
                ],
                "sample_id": "213543331",
                "type": "control",
            }
        ],
    }
    assert worklist == expected
