# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import driverng


parser_class = "cobas_pure"


def test_decode_query_sequence_mode(self):
    worklist = self.decode(
        """
        MSH|^~\\&|cpure||host||20160724080600+0200||QBP^Q11^QBP_Q11|1233|P|2.5.1|||NE|AL|UNICODE UTF-8|||LAB-27R^ROCHE
    QPD|INISEQ^^99ROC|query1233|123|50001|1|||||SERPLAS^^99ROC|SC^^99ROC|R
    RCP|I|1|R^^HL70394
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "123", "rack": "50001-1"}],
    }
    assert worklist == expected


def test_decode_query_barcode_mode(self):
    worklist = self.decode(
        """
        MSH|^~\\&|cpure||host||20160724080600+0200||QBP^Q11^QBP_Q11|1234|P|2.5.1|||NE|AL|UNICODE UTF-8|||LAB-27R^ROCHE
    QPD|INIBAR^^99ROC|query1234|10001|50001|1|||||SERPLAS^^99ROC|SC^^99ROC|S
    RCP|I|1|S^^HL70394
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "10001", "rack": "50001-1"}],
    }
    assert worklist == expected


def test_decode_quantitative_c_303_result(self):
    worklist = self.decode(
        """
        MSH|^~\\&|cpure||host||20180222150842+0100||OUL^R22^OUL_R22|97|P|2.5.1|||NE|AL||UNICODE·UTF-8|||LAB-29^IHE
    PID|||||^^^^^^U|||U
    SPM|1|022&BARCODE||SERPLAS^^99ROC|||||||P^^HL70369|||~~~~||||||||||PSCO^^99ROC|||SC^^99ROC
    SAC|||022^BARCODE|||||||50120|2||||||||||||||||||^1^:^1
    OBR|1|""||20490^^99ROC|||||||
    ORC|SC||||CM
    TQ1|||||||||R^^HL70485
    OBX|1|NM|20490^20490^99ROC^^^IHELAW|1|32.2|mg/L^^99ROC||N^^HL70078|||F|||||Admin~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20180222150842||||||||||RSLT
    OBX|2|CE|20490^20490^99ROC^^^IHELAW|1|-1^^99ROC|||N^^HL70078|||F|||||Admin~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20180222150842||||||||||RSLT
    TCD|20490^^99ROC|^1^:^1
    INV|2049001|OK^^HL70383~CURRENT^^99ROC|R1|514|1|8||||||20181030||||256616
    INV|2049001|OK^^HL70383~CURRENT^^99ROC|R3|514|1|8||||||20181030||||256616
    OBX|3|DTM|PT^Pipetting_Time^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|20180222145824|||N^^HL70078|||F|||||Admin~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20180222150842||||||||||RSLT
    OBX|4|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|23|||N^^HL70078|||F|||||Admin~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20180222150842||||||||||RSLT
    OBX|5|EI|QCTID^QC·Test·ID^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|62~67|||N^^HL70078|||F|||||Admin~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20180222150842||||||||||RSLT
    OBX|6|CE|QCSTATE^QC·Status^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|2^^99ROC|||N^^HL70078|||F|||||Admin~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20180222150842||||||||||RSLT
    OBX|7|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|0.300·-·350|||N^^HL70078|||F|||||Admin~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20180222150842||||||||||RSLT
    OBX|8|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|-99999·-·999999|||N^^HL70078|||F|||||Admin~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20180222150842||||||||||RSLT
    OBX|9|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|-99999·-·999999|||N^^HL70078|||F|||||Admin~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20180222150842||||||||||RSLT
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "50120-2",
                "results": [
                    {
                        "code": "20490",
                        "date": datetime.datetime(2018, 2, 22, 15, 8, 42),
                        "flags": ["N"],
                        "instrument": "c303",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2018, 10, 30, 0, 0),
                                "lot": "256616",
                                "name": "2049001-R1",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2018, 10, 30, 0, 0),
                                "lot": "256616",
                                "name": "2049001-R3",
                                "type": "reagent",
                            },
                        ],
                        "operator": "Admin",
                        "unit": "mg/L",
                        "value": "32.2",
                    },
                    {
                        "code": "20490_I",
                        "date": datetime.datetime(2018, 2, 22, 15, 8, 42),
                        "instrument": "c303",
                        "operator": "Admin",
                        "value": "-1",
                    },
                ],
                "sample_id": "022",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_quantitative_ise_result(self):
    worklist = self.decode(
        """
        MSH|^~\\&|cpure||host||20180222145720+0100||OUL^R22^OUL_R22|81|P|2.5.1|||NE|AL||UNICODE·UTF-8|||LAB-29^IHE
    PID|||||^^^^^^U|||U
    SPM|1|021&BARCODE||SERPLAS^^99ROC|||||||P^^HL70369|||~~~~||||||||||PSCO^^99ROC|||SC^^99ROC
    SAC|||021^BARCODE|||||||50120|1||||||||||||||||||^1^:^1
    OBR|1|""||29070^^99ROC|||||||
    ORC|SC||||CM
    TQ1|||||||||R^^HL70485
    OBX|1|NM|29070^29070^99ROC^^^IHELAW|1|111.1|mmol/L^^99ROC||N^^HL70078|||F|||||Admin~REALTIME||ISE^ROCHE~1111^ROCHE|20180222145720||||||||||RSLT
    OBX|2|CE|29070^29070^99ROC^^^IHELAW|1|^^99ROC|||N^^HL70078|||F|||||Admin~REALTIME||ISE^ROCHE~1111^ROCHE|20180222145720||||||||||RSLT
    TCD|29070^^99ROC|^1^:^1
    INV|2999002|OK^^HL70383~CURRENT^^99ROC|IS|368|1|1||||||20190831||||306893
    INV|2999001|OK^^HL70383~CURRENT^^99ROC|DIL|644|1|1||||||20190930||||306894
    INV|2999003|OK^^HL70383~CURRENT^^99ROC|REF|400|1|1||||||20200430||||310025
    OBX|3|DTM|PT^Pipetting_Time^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|20180222145654|||N^^HL70078|||F|||||Admin~REALTIME||ISE^ROCHE~1111^ROCHE|20180222145720||||||||||RSLT
    OBX|4|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|37|||N^^HL70078|||F|||||Admin~REALTIME||ISE^ROCHE~1111^ROCHE|20180222145720||||||||||RSLT
    OBX|5|EI|QCTID^QC·Test·ID^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|47~35|||N^^HL70078|||F|||||Admin~REALTIME||ISE^ROCHE~1111^ROCHE|20180222145720||||||||||RSLT
    OBX|6|CE|QCSTATE^QC·Status^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|1^^99ROC|||N^^HL70078|||F|||||Admin~REALTIME||ISE^ROCHE~1111^ROCHE|20180222145720||||||||||RSLT
    OBX|7|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|80.0·-·180|||N^^HL70078|||F|||||Admin~REALTIME||ISE^ROCHE~1111^ROCHE|20180222145720||||||||||RSLT
    OBX|8|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|-99999·-·999999|||N^^HL70078|||F|||||Admin~REALTIME||ISE^ROCHE~1111^ROCHE|20180222145720||||||||||RSLT
    OBX|9|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^Other·Supplemental^IHELAW|1|-99999·-·999999|||N^^HL70078|||F|||||Admin~REALTIME||ISE^ROCHE~1111^ROCHE|20180222145720||||||||||RSLT
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "50120-1",
                "results": [
                    {
                        "code": "29070",
                        "date": datetime.datetime(2018, 2, 22, 14, 57, 20),
                        "flags": ["N"],
                        "instrument": "ISE",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2019, 8, 31, 0, 0),
                                "lot": "306893",
                                "name": "2999002-IS",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2019, 9, 30, 0, 0),
                                "lot": "306894",
                                "name": "2999001-DIL",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2020, 4, 30, 0, 0),
                                "lot": "310025",
                                "name": "2999003-REF",
                                "type": "reagent",
                            },
                        ],
                        "operator": "Admin",
                        "unit": "mmol/L",
                        "value": "111.1",
                    }
                ],
                "sample_id": "021",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_quantitative_with_qualitative_result(self):
    worklist = self.decode(
        """
        MSH|^~\\&|cpure||host||20180222145720+0100||OUL^R22^OUL_R22|81|P|2.5.1|||NE|AL||UNICODE·UTF-8|||LAB-29^IHE
    PID|||||^^^^^^U|||U
    SPM|1|1&SEQUENCE||SERPLAS^^99ROC|||||||P^^HL70369|||~~~~|||||||||||||SC^^99ROC
    SAC|||1^SEQUENCE|||||||50001|1||||||||||||||||||^1^:^1
    OBR|1|""||11151^^99ROC|||||||
    ORC|SC||||CM
    TQ1|||||||||R^^HL70485
    OBX|1|NM|11151^11151^99ROC^^^IHELAW|1|0.917|U/mL^^99ROC||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    OBX|2|CE|11151^11151^99ROC^^^IHELAW|1|^^99ROC|||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    TCD|11151^^99ROC|^1^:^1
    INV|1011151|OK^^HL70383~CURRENT^^99ROC|ASY|211|1|8||||||||||212218
    OBX|3|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180407164425|||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    OBX|4|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|1066|||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    OBX|5|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1||||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    OBX|6|CE|QCSTATE^QC Status^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|0^^99ROC|||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    OBX|7|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|0.150 -500|||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    OBX|8|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1|-99999-999999|||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    OBX|9|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^Other Supplemental^IHELAW|1|-99999 -999999|||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    OBX|10|NM|11151_EFS^11151_EFS^99ROC^S_RAW^RawSupplemental^IHELAW|1|2131.9690|COUNT^^99ROC||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    OBX|11|NM|11151_EFV^11151_EFV^99ROC^S_RAW^RawSupplemental^IHELAW|1|0.6080|COUNT^^99ROC||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    OBX|12|NM|11151_EFC^11151_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-0.6400|COUNT^^99ROC||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    OBX|13|NM|11151_PMT^11151_PMT^99ROC^S_RAW^RawSupplemental^IHELAW|1|40080|COUNT^^99ROC||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407164532||||||||||RSLT
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "50001-1",
                "results": [
                    {
                        "code": "11151",
                        "date": datetime.datetime(2018, 4, 7, 16, 45, 32),
                        "flags": ["102"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "212218",
                                "name": "1011151-ASY",
                                "type": "reagent",
                            }
                        ],
                        "operator": "cobas",
                        "unit": "U/mL",
                        "value": "0.917",
                    },
                    {
                        "code": "11151_EFS",
                        "date": datetime.datetime(2018, 4, 7, 16, 45, 32),
                        "flags": ["102"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "2131.9690",
                    },
                    {
                        "code": "11151_EFV",
                        "date": datetime.datetime(2018, 4, 7, 16, 45, 32),
                        "flags": ["102"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "0.6080",
                    },
                    {
                        "code": "11151_EFC",
                        "date": datetime.datetime(2018, 4, 7, 16, 45, 32),
                        "flags": ["102"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "-0.6400",
                    },
                    {
                        "code": "11151_PMT",
                        "date": datetime.datetime(2018, 4, 7, 16, 45, 32),
                        "flags": ["102"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "40080",
                    },
                ],
                "sample_id": "1",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_e_flow_result(self):
    worklist = self.decode(
        """
        MSH|^~\\&|cpure||host||20180408022608+0900||OUL^R22^OUL_R22|178|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|1&SEQUENCE||SERPLAS^^99ROC|||||||P^^HL70369|||~~~~|||||||||||||SC^^99ROC
    SAC|||1^SEQUENCE|||||||50001|1||||||||||||||||||^1^:^1
    OBR|1|""||12000^^99ROC|||||||
    ORC|SC||||CM
    TQ1|||||||||R^^HL70485
    OBX|1|NM|12000^12000^99ROC^^^IHELAW|1||^^99ROC||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022608||||||||||RSLT
    OBX|2|CE|12000^12000^99ROC^^^IHELAW|1|^^99ROC|||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022608||||||||||RSLT
    OBX|3|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1||||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022608||||||||||RSLT
    OBX|4|CE|QCSTATE^QC Status^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|0^^99ROC|||102^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022608||||||||||RSLT
    OBX|5|NM|11013^11013^99ROC^^^IHELAW|1|*******|COI^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|6|CE|11013^11013^99ROC^^^IHELAW|1|^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    TCD|11013^^99ROC|^1^:^1
    INV|1311013|OK^^HL70383~CURRENT^^99ROC|ASY|5|1|15||||||20181231||||188548
    OBX|7|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180408022342|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|8|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|144|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|9|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|3205~3844|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|10|CE|QCSTATE^QC Status^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|2^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|11|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|0.000000 -0.000000|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|12|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1|-99999-999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|13|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|-99999 -999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|14|NM|11013_EFS^11013_EFS^99ROC^S_RAW^RawSupplemental^IHELAW|1|2131.9690|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|15|NM|11013_EFV^11013_EFV^99ROC^S_RAW^RawSupplemental^IHELAW|1|0.6080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|16|NM|11013_EFC^11013_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-0.6400|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|17|NM|11013_PMT^11013_PMT^99ROC^S_RAW^RawSupplemental^IHELAW|1|40080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|18|NM|11013^11013^99ROC^^^IHELAW|1|*******|COI^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|19|CE|11013^11013^99ROC^^^IHELAW|1|^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    TCD|11013^^99ROC|^1^:^1
    INV|1311013|OK^^HL70383~CURRENT^^99ROC|ASY|5|1|15||||||20181231||||188548
    OBX|20|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180408022455|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|21|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|144|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|22|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|3205~3844|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|23|CE|QCSTATE^QC Status^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|2^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|24|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|0.000000 -0.000000|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|25|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1|-99999-999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|26|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|-99999 -999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|27|NM|11013_EFS^11013_EFS^99ROC^S_RAW^RawSupplemental^IHELAW|1|2131.9690|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|28|NM|11013_EFV^11013_EFV^99ROC^S_RAW^RawSupplemental^IHELAW|1|0.6080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|29|NM|11013_EFC^11013_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-0.6400|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|30|NM|11013_PMT^11013_PMT^99ROC^S_RAW^RawSupplemental^IHELAW|1|40080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|31|NM|11013^11013^99ROC^^^IHELAW|1|*******|COI^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|32|CE|11013^11013^99ROC^^^IHELAW|1|^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    TCD|11013^^99ROC|^1^:^1
    INV|1311013|OK^^HL70383~CURRENT^^99ROC|ASY|5|1|15||||||20181231||||188548
    OBX|33|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180408022457|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|34|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|144|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|35|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|3205~3844|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|36|CE|QCSTATE^QC Status^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|2^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|37|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|0.000000 -0.000000|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|38|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1|-99999-999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|39|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|-99999 -999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|40|NM|11013_EFS^11013_EFS^99ROC^S_RAW^RawSupplemental^IHELAW|1|2131.9690|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|41|NM|11013_EFV^11013_EFV^99ROC^S_RAW^RawSupplemental^IHELAW|1|0.6080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|42|NM|11013_EFC^11013_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-0.6400|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|43|NM|11013_PMT^11013_PMT^99ROC^S_RAW^RawSupplemental^IHELAW|1|40080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022608||||||||||RSLT
    OBX|44|NM|11014^11014^99ROC^^^IHELAW|1|*******|COI^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|45|CE|11014^11014^99ROC^^^IHELAW|1|^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    TCD|11014^^99ROC|^1^:^1
    INV|1311014|OK^^HL70383~CURRENT^^99ROC|ASY|5|1|16||||||20181231||||188548
    OBX|46|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180408022341|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|47|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|147|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|48|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|3206~3845|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|49|CE|QCSTATE^QC Status^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|2^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|50|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|0.000000 -0.000000|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|51|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1|-99999-999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|52|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|-99999 -999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|53|NM|11014_EFS^11014_EFS^99ROC^S_RAW^RawSupplemental^IHELAW|1|2131.9690|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|54|NM|11014_EFV^11014_EFV^99ROC^S_RAW^RawSupplemental^IHELAW|1|0.6080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|55|NM|11014_EFC^11014_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-0.6400|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|56|NM|11014_PMT^11014_PMT^99ROC^S_RAW^RawSupplemental^IHELAW|1|40080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022451||||||||||RSLT
    OBX|57|NM|11014^11014^99ROC^^^IHELAW|1|*******|COI^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|58|CE|11014^11014^99ROC^^^IHELAW|1|^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    TCD|11014^^99ROC|^1^:^1
    INV|1311014|OK^^HL70383~CURRENT^^99ROC|ASY|5|1|16||||||20181231||||188548
    OBX|59|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180408022454|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|60|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|147|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|61|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|3206~3845|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|62|CE|QCSTATE^QC Status^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|2^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|63|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|0.000000 -0.000000|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|64|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1|-99999-999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|65|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|-99999 -999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|66|NM|11014_EFS^11014_EFS^99ROC^S_RAW^RawSupplemental^IHELAW|1|2131.9690|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|67|NM|11014_EFV^11014_EFV^99ROC^S_RAW^RawSupplemental^IHELAW|1|0.6080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|68|NM|11014_EFC^11014_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-0.6400|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|69|NM|11014_PMT^11014_PMT^99ROC^S_RAW^RawSupplemental^IHELAW|1|40080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022606||||||||||RSLT
    OBX|70|NM|11014^11014^99ROC^^^IHELAW|1|*******|COI^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    OBX|71|CE|11014^11014^99ROC^^^IHELAW|1|^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    TCD|11014^^99ROC|^1^:^1
    INV|1311014|OK^^HL70383~CURRENT^^99ROC|ASY|5|1|16||||||20181231||||188548
    OBX|72|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180408022456|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    OBX|73|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|147|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    OBX|74|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|3206~3845|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    OBX|75|CE|QCSTATE^QC Status^99ROC^S_OTHER^Other Supplemental^IHELAW|1|2^^99ROC|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~12^ROCHE|20180408022607||||||||||RSLT
    OBX|76|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|0.000000 -0.000000|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    OBX|77|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1|-99999-999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    OBX|78|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|-99999 -999999|||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    OBX|79|NM|11014_EFS^11014_EFS^99ROC^S_RAW^RawSupplemental^IHELAW|1|2131.9690|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    OBX|80|NM|11014_EFV^11014_EFV^99ROC^S_RAW^RawSupplemental^IHELAW|1|0.6080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    OBX|81|NM|11014_EFC^11014_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-0.6400|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    OBX|82|NM|11014_PMT^11014_PMT^99ROC^S_RAW^RawSupplemental^IHELAW|1|40080|COUNT^^99ROC||102^^99ROC~SUP^^99ROC|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180408022607||||||||||RSLT
    OBX|83|NM|AHIV_01^AHIV_01^99ROC^^^IHELAW|1|1.00|COI^^99ROC||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|84|CE|AHIV_01^AHIV_01^99ROC^^^IHELAW|1|^^99ROC|||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|85|NM|AHIV_02^AHIV_02^99ROC^^^IHELAW|1|1.00|COI^^99ROC||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|86|CE|AHIV_02^AHIV_02^99ROC^^^IHELAW|1|^^99ROC|||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|87|NM|AHIV_03^AHIV_03^99ROC^^^IHELAW|1|1.00|COI^^99ROC||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|88|CE|AHIV_03^AHIV_03^99ROC^^^IHELAW|1|^^99ROC|||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|89|NM|HIVAG_01^HIVAG_01^99ROC^^^IHELAW|1|1.00|COI^^99ROC||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|90|CE|HIVAG_01^HIVAG_01^99ROC^^^IHELAW|1|^^99ROC|||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|91|NM|HIVAG_02^HIVAG_02^99ROC^^^IHELAW|1|1.00|COI^^99ROC||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|92|CE|HIVAG_02^HIVAG_02^99ROC^^^IHELAW|1|^^99ROC|||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|93|NM|HIVAG_03^HIVAG_03^99ROC^^^IHELAW|1|1.00|COI^^99ROC||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|94|CE|HIVAG_03^HIVAG_03^99ROC^^^IHELAW|1|^^99ROC|||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|95|NM|HIVDuo_01^HIVDuo_01^99ROC^^^IHELAW|1|1.41|COI^^99ROC||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|96|CE|HIVDuo_01^HIVDuo_01^99ROC^^^IHELAW|1|^^99ROC|||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|97|NM|HIVDuo_02^HIVDuo_02^99ROC^^^IHELAW|1|1.41|COI^^99ROC||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|98|CE|HIVDuo_02^HIVDuo_02^99ROC^^^IHELAW|1|^^99ROC|||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|99|NM|HIVDuo_03^HIVDuo_03^99ROC^^^IHELAW|1|1.41|COI^^99ROC||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|100|CE|HIVDuo_03^HIVDuo_03^99ROC^^^IHELAW|1|^^99ROC|||||||||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|||||||||||RSLT
    OBX|101|NM|12000^12000^99ROC^^^IHELAW|1|""|||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022451||||||||||RSLT
    OBX|102|CE|12000^12000^99ROC^^^IHELAW|1||||^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022451||||||||||RSLT
    INV|12000|OK^^HL70383~|KIT|57|1|""||||||20181231||||188548
    OBX|103|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180408022341|||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022451||||||||||RSLT
    OBX|104|EI|CombID^Measurement Cell Test Combination ID^99ROC^^^S_OTHER^OtherSupplemental^IHELAW|1|1|||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022451||||||||||RSLT
    OBX|105|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1||||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022451||||||||||RSLT
    OBX|106|NM|12000^12000^99ROC^^^IHELAW|1|""|||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022606||||||||||RSLT
    OBX|107|CE|12000^12000^99ROC^^^IHELAW|1||||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022606||||||||||RSLT
    INV|12000|OK^^HL70383~|KIT|57|1|""||||||20181231||||188548
    OBX|108|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180408022454|||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022606||||||||||RSLT
    OBX|109|EI|CombID^Measurement Cell Test Combination ID^99ROC^^^S_OTHER^OtherSupplemental^IHELAW|1|1|||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022606||||||||||RSLT
    OBX|110|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1||||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022606||||||||||RSLT
    OBX|111|NM|12000^12000^99ROC^^^IHELAW|1|""|||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022608||||||||||RSLT
    OBX|112|CE|12000^12000^99ROC^^^IHELAW|1||||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022608||||||||||RSLT
    INV|12000|OK^^HL70383~|KIT|57|1|""||||||20181231||||188548
    OBX|113|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180408022456|||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022608||||||||||RSLT
    OBX|114|EI|CombID^Measurement Cell Test Combination ID^99ROC^^^S_OTHER^OtherSupplemental^IHELAW|1|1|||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022608||||||||||RSLT
    OBX|115|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1||||102^^99ROC|||B|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE|20180408022608||||||||||RSLT
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "50001-1",
                "results": [
                    {
                        "code": "12000",
                        "date": datetime.datetime(2018, 4, 8, 2, 26, 8),
                        "flags": ["102"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "",
                        "value": "",
                    },
                    {
                        "code": "11013",
                        "date": datetime.datetime(2018, 4, 8, 2, 24, 51),
                        "flags": ["102"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2018, 12, 31, 0, 0),
                                "lot": "188548",
                                "name": "1311013-ASY",
                                "type": "reagent",
                            }
                        ],
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "*******",
                    },
                    {
                        "code": "11013_EFS",
                        "date": datetime.datetime(2018, 4, 8, 2, 24, 51),
                        "flags": ["102"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "2131.9690",
                    },
                    {
                        "code": "11013_EFV",
                        "date": datetime.datetime(2018, 4, 8, 2, 24, 51),
                        "flags": ["102"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "0.6080",
                    },
                    {
                        "code": "11013_EFC",
                        "date": datetime.datetime(2018, 4, 8, 2, 24, 51),
                        "flags": ["102"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "-0.6400",
                    },
                    {
                        "code": "11013_PMT",
                        "date": datetime.datetime(2018, 4, 8, 2, 24, 51),
                        "flags": ["102"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2018, 12, 31, 0, 0),
                                "lot": "188548",
                                "name": "1311013-ASY",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2018, 12, 31, 0, 0),
                                "lot": "188548",
                                "name": "1311013-ASY",
                                "type": "reagent",
                            },
                        ],
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "40080",
                    },
                    {
                        "code": "11014",
                        "date": datetime.datetime(2018, 4, 8, 2, 24, 51),
                        "flags": ["102"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2018, 12, 31, 0, 0),
                                "lot": "188548",
                                "name": "1311014-ASY",
                                "type": "reagent",
                            }
                        ],
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "*******",
                    },
                    {
                        "code": "11014_EFS",
                        "date": datetime.datetime(2018, 4, 8, 2, 24, 51),
                        "flags": ["102"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "2131.9690",
                    },
                    {
                        "code": "11014_EFV",
                        "date": datetime.datetime(2018, 4, 8, 2, 24, 51),
                        "flags": ["102"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "0.6080",
                    },
                    {
                        "code": "11014_EFC",
                        "date": datetime.datetime(2018, 4, 8, 2, 24, 51),
                        "flags": ["102"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "-0.6400",
                    },
                    {
                        "code": "11014_PMT",
                        "date": datetime.datetime(2018, 4, 8, 2, 24, 51),
                        "flags": ["102"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2018, 12, 31, 0, 0),
                                "lot": "188548",
                                "name": "1311014-ASY",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2018, 12, 31, 0, 0),
                                "lot": "188548",
                                "name": "1311014-ASY",
                                "type": "reagent",
                            },
                        ],
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "40080",
                    },
                    {
                        "code": "AHIV_01",
                        "date": None,
                        "flags": [],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "1.00",
                    },
                    {
                        "code": "AHIV_02",
                        "date": None,
                        "flags": [],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "1.00",
                    },
                    {
                        "code": "AHIV_03",
                        "date": None,
                        "flags": [],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "1.00",
                    },
                    {
                        "code": "HIVAG_01",
                        "date": None,
                        "flags": [],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "1.00",
                    },
                    {
                        "code": "HIVAG_02",
                        "date": None,
                        "flags": [],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "1.00",
                    },
                    {
                        "code": "HIVAG_03",
                        "date": None,
                        "flags": [],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "1.00",
                    },
                    {
                        "code": "HIVDuo_01",
                        "date": None,
                        "flags": [],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "1.41",
                    },
                    {
                        "code": "HIVDuo_02",
                        "date": None,
                        "flags": [],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "1.41",
                    },
                    {
                        "code": "HIVDuo_03",
                        "date": None,
                        "flags": [],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2018, 12, 31, 0, 0),
                                "lot": "188548",
                                "name": "12000-KIT",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2018, 12, 31, 0, 0),
                                "lot": "188548",
                                "name": "12000-KIT",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2018, 12, 31, 0, 0),
                                "lot": "188548",
                                "name": "12000-KIT",
                                "type": "reagent",
                            },
                        ],
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "1.41",
                    },
                ],
                "sample_id": "1",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_formula_calculated(self):
    worklist = self.decode(
        """
        MSH|^~¥&|cpure||Host||20191120105710+0900||OUL^R22^OUL_R22|37|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|1&SEQUENCE||SERPLAS^^99ROC|||||||P^^HL70369|||~~~~||||||||||PSCO^^99ROC
    SAC|||1^SEQUENCE|||||||50001|1||||||||||||||||||^1^:^1
    OBR|1|""||31001^^99ROC|||||||
    ORC|SC||||CM
    TQ1|||||||||R^^HL70485
    OBX|1|NM|31001^31001^99ROC^^^IHELAW|1|341|test^^99ROC||37^^99ROC~112^^99ROC|||F|||||~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20191120105710||||||||||RSLT
    OBX|2|CE|31001^31001^99ROC^^^IHELAW|1|^^99ROC|||37^^99ROC~112^^99ROC|||F|||||~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20191120105710||||||||||RSLT
    TCD|31001^^99ROC|^1^:^1
    OBX|3|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1||||37^^99ROC~112^^99ROC|||F|||||~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20191120105710||||||||||RSLT
    OBX|4|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1||||37^^99ROC~112^^99ROC|||F|||||~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20191120105710||||||||||RSLT
    OBX|5|EI|QCTID^QC Test ID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1||||37^^99ROC~112^^99ROC|||F|||||~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20191120105710||||||||||RSLT
    OBX|6|CE|QCSTATE^QC Status^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|0^^99ROC|||37^^99ROC~112^^99ROC|||F|||||~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20191120105710||||||||||RSLT
    OBX|7|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^OtherSupplemental^IHELAW|1||||37^^99ROC~112^^99ROC|||F|||||~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20191120105710||||||||||RSLT
    OBX|8|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1| -|||37^^99ROC~112^^99ROC|||F|||||~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20191120105710||||||||||RSLT
    OBX|9|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^Other Supplemental^IHELAW|1|-9999999 -9999999|||37^^99ROC~112^^99ROC|||F|||||~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20191120105710||||||||||RSLT
    OBX|10|NA|31001_ABS^31001_ABS^99ROC^S_GRAPH^SupplementalGraph^IHELAW|1|0.0000^0^0~0^0~0.0000^0.0000^0.0000^0.0000~0.0000^0.0000^0.0000^0.0000~0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000~0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000^0.0000|ABS^^99ROC||37^^99ROC~112^^99ROC|||F|||||~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20191120105710||||||||||RSLT
    OBX|11|NA|31001_RCV^31001_RCV^99ROC^S_GRAPH^SupplementalGraph^IHELAW|1|~~~^^^^^^^^^~^^|RCV^^99ROC||37^^99ROC~112^^99ROC|||F|||||~REALTIME||c303^ROCHE~3333^ROCHE~1^ROCHE|20191120105710||||||||||RSLT
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "50001-1",
                "results": [
                    {
                        "code": "31001",
                        "date": datetime.datetime(2019, 11, 20, 10, 57, 10),
                        "flags": ["37"],
                        "instrument": "c303",
                        "operator": "",
                        "unit": "test",
                        "value": "341",
                    }
                ],
                "sample_id": "1",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_qc_result(self):
    worklist = self.decode(
        """
        MSH|^~\\&|cpure||host||20180407181537+0900||OUL^R22^OUL_R22|81|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|25001&CONTROL||SERPLAS^^99ROC|||||||Q^^HL70369|||~~~~|||||20191231||||||||SC^^99ROC
    SAC|||25001^CONTROL|||||||000001|0
    OBR|3|""||12018^^99ROC|||||||
    ORC|SC||||CM
    TQ1|||||||||R^^HL70485
    OBX|1|NM|12018^12018^99ROC^^^IHELAW|1|0.958|COI^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~0^ROCHE|20180407181536||3782||||||||RSLT
    OBX|2|CE|12018^12018^99ROC^^^IHELAW|1|^^99ROC|||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~0^ROCHE|20180407181536||3782||||||||RSLT
    INV|12018|OK^^HL70383~CURRENT^^99ROC|KIT|57|1|""||||||20181231||||188548
    OBX|3|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180407181422|||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~0^ROCHE|20180407181536||3782||||||||RSLT
    OBX|4|EI|CombID^Measurement Cell Test Combination ID^99ROC^^^S_OTHER^OtherSupplemental^IHELAW|1|1|||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~0^ROCHE|20180407181536||3782||||||||RSLT
    OBX|5|NM|11014^11014^99ROC^^^IHELAW|1|0.677|COI^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181535||3780||||||||RSLT
    OBX|6|CE|11014^11014^99ROC^^^IHELAW|1|-1^^99ROC|||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181535||3780||||||||RSLT
    TCD|11014^^99ROC|^1^:^1
    INV|1311014|OK^^HL70383~CURRENT^^99ROC|ASY|5|1|16||||||||||188548
    OBX|7|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180407181422|||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181535||3780||||||||RSLT
    OBX|8|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|146|||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181535||3780||||||||RSLT
    OBX|9|NM|QC_TARGET^QC_TARGET^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|100|COI^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181535||3780||||||||RSLT
    OBX|10|NM|QC_SD_RANGE^QC_SD_RANGE^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|100|COI^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181535||3780||||||||RSLT
    OBX|11|NM|11014_EFS^11014_EFS^99ROC^S_RAW^RawSupplemental^IHELAW|1|2131.9690|COUNT^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181535||||||||||RSLT
    OBX|12|NM|11014_EFV^11014_EFV^99ROC^S_RAW^RawSupplemental^IHELAW|1|0.6080|COUNT^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181535||||||||||RSLT
    OBX|13|NM|11014_EFC^11014_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-0.6400|COUNT^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181535||||||||||RSLT
    OBX|14|NM|11014_PMT^11014_PMT^99ROC^S_RAW^RawSupplemental^IHELAW|1|40080|COUNT^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181535||||||||||RSLT
    OBX|15|NM|11013^11013^99ROC^^^IHELAW|1|0.677|COI^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181536||3781||||||||RSLT
    OBX|16|CE|11013^11013^99ROC^^^IHELAW|1|-1^^99ROC|||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181536||3781||||||||RSLT
    TCD|11013^^99ROC|^1^:^1
    INV|1311013|OK^^HL70383~CURRENT^^99ROC|ASY|5|1|15||||||||||188548
    OBX|17|DTM|PT^Pipetting_Time^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|20180407181423|||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181536||3781||||||||RSLT
    OBX|18|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|145|||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181536||3781||||||||RSLT
    OBX|19|NM|QC_TARGET^QC_TARGET^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|100|COI^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181536||3781||||||||RSLT
    OBX|20|NM|QC_SD_RANGE^QC_SD_RANGE^99ROC^S_OTHER^OtherSupplemental^IHELAW|1|100|COI^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181536||3781||||||||RSLT
    OBX|21|NM|11013_EFS^11013_EFS^99ROC^S_RAW^RawSupplemental^IHELAW|1|2131.9690|COUNT^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181536||||||||||RSLT
    OBX|22|NM|11013_EFV^11013_EFV^99ROC^S_RAW^RawSupplemental^IHELAW|1|0.6080|COUNT^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181536||||||||||RSLT
    OBX|23|NM|11013_EFC^11013_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-0.6400|COUNT^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181536||||||||||RSLT
    OBX|24|NM|11013_PMT^11013_PMT^99ROC^S_RAW^RawSupplemental^IHELAW|1|40080|COUNT^^99ROC||N^^HL70078|||F|||||cobas~REALTIME||e402^ROCHE~4444^ROCHE~1^ROCHE|20180407181536||||||||||RSLT
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "000001",
                "results": [
                    {
                        "code": "12018",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 36),
                        "flags": ["N"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2018, 12, 31, 0, 0),
                                "lot": "188548",
                                "name": "12018-KIT",
                                "type": "reagent",
                            }
                        ],
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "0.958",
                    },
                    {
                        "code": "11014",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 35),
                        "flags": ["N"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "188548",
                                "name": "1311014-ASY",
                                "type": "reagent",
                            }
                        ],
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "0.677",
                    },
                    {
                        "code": "11014_I",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 35),
                        "instrument": "e402",
                        "operator": "cobas",
                        "value": "-1",
                    },
                    {
                        "code": "QC_TARGET",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 35),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "100",
                    },
                    {
                        "code": "QC_SD_RANGE",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 35),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "100",
                    },
                    {
                        "code": "11014_EFS",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 35),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "2131.9690",
                    },
                    {
                        "code": "11014_EFV",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 35),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "0.6080",
                    },
                    {
                        "code": "11014_EFC",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 35),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "-0.6400",
                    },
                    {
                        "code": "11014_PMT",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 35),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "40080",
                    },
                    {
                        "code": "11013",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 36),
                        "flags": ["N"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "188548",
                                "name": "1311013-ASY",
                                "type": "reagent",
                            }
                        ],
                        "operator": "cobas",
                        "unit": "COI",
                        "value": "0.677",
                    },
                    {
                        "code": "11013_I",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 36),
                        "instrument": "e402",
                        "operator": "cobas",
                        "value": "-1",
                    },
                    {
                        "code": "11013_EFS",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 36),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "2131.9690",
                    },
                    {
                        "code": "11013_EFV",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 36),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "0.6080",
                    },
                    {
                        "code": "11013_EFC",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 36),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "-0.6400",
                    },
                    {
                        "code": "11013_PMT",
                        "date": datetime.datetime(2018, 4, 7, 18, 15, 36),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "cobas",
                        "unit": "COUNT",
                        "value": "40080",
                    },
                ],
                "sample_id": "000001",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_order(self):
    message = self.order(context={"control_id": "1237"})
    expected = """
    MSH|^~\\&|||||20130715141912||OML^O33^OML_O33|1237|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-28R^ROCHE
    PID|1||036666666||^^^^^^U||20110513|F
    SPM|1|123456&BARCODE||SERPLAS^^99ROC|||||||P^^HL70369|||NYME~ANO~036666666~NJF~Prescriber|||||||||||||SC^^99ROC
    SAC|1||123456||||||||||||||||||||||||||^1^1
    ORC|NW||||||||20130603110342
    TQ1|1||||||||S^^HL70485
    OBR|1|78910||A^^99ROC
    TCD|A^^99ROC|^1^:^10
    ORC|NW||||||||20130603110342
    TQ1|1||||||||S^^HL70485
    OBR|1|78910||B^^99ROC
    TCD|B^^99ROC|^1^:^1
    SPM|2|123456&BARCODE||SERPLAS^^99ROC|||||||P^^HL70369|||NYME~ANO~036666666~NJF~Prescriber|||||||||||||SC^^99ROC
    SAC|1||123456||||||||||||||||||||||||||^1^1
    ORC|CA||||||||20130603110342
    TQ1|1||||||||S^^HL70485
    OBR|1|78910||C^^99ROC
    TCD|C^^99ROC|^1^:^1
    """
    assert message == expected


def test_encode_order(self):
    worklist = driverng.Worklist(
        context="order",
        samples=[
            {
                "sample_id": "10001",
                "request_id": "4711",
                "type": "patient",
                "orders": [
                    {"code": "8714", "action": "add", "dilution": "1:2"},
                    {"code": "8717", "action": "add", "dilution": "1:1"},
                ],
                "birthdate": datetime.datetime(1927, 10, 6),
                "sex": "M",
                "patient_id": "123",
                "specimen": "SERPLAS",
                "priority": "R",
                "request_date": datetime.datetime(2016, 7, 24, 8, 5, 12),
            }
        ],
    )
    message = self.order(worklist=worklist, context={"control_id": "1237"})
    expected = """
    MSH|^~\\&|||||20130715141912||OML^O33^OML_O33|1237|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-28R^ROCHE
    PID|1||123||^^^^^^U||19271006|M
    SPM|1|10001&BARCODE||SERPLAS^^99ROC|||||||P^^HL70369|||~~123~~|||||||||||||SC^^99ROC
    SAC|1||10001||||||||||||||||||||||||||^1^1
    ORC|NW||||||||20160724080512
    TQ1|1||||||||R^^HL70485
    OBR|1|4711||8714^^99ROC
    TCD|8714^^99ROC|^1^:^2
    ORC|NW||||||||20160724080512
    TQ1|1||||||||R^^HL70485
    OBR|1|4711||8717^^99ROC
    TCD|8717^^99ROC|^1^:^1
    """
    assert message == expected


def test_void_order(self):
    message = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
            MSH|^~\\&|cpure||Host||20130715141912||QBP^Q11^QBP_Q11|9|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-27R^ROCHE
        QPD|INIBAR^^99ROC|0|10001|51201|1|||||SERPLAS^^99ROC|SC^^99ROC|R
        RCP|I|1|R^HL70394
            """
            ),
            "control_id": "199",
        },
    )
    expected = """
        MSH|^~\\&|||||20130715141912||OML^O33^OML_O33|199|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-28R^ROCHE
    SPM|1|10001&BARCODE||SERPLAS^^99ROC|||||||P^^HL70369||||||||||||||||SC^^99ROC
    SAC|1||10001^BARCODE|||||||51201|1||||||||||||||||||^1^1
    ORC|DC||||||||20130715141912
    """
    assert message == expected


def test_void_order_in_non_racks(self):
    message = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
            MSH|^~\\&|cobas pure||Host||20220907153022+0900||QBP^Q11^QBP_Q11|1802|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-27R^ROCHE
        QPD|INIBAR^^99ROC|0|1453848101|50016|3|||||""|SC^^99ROC|R
        RCP|I|1|R^HL70394
            """
            ),
            "control_id": "199",
        },
    )
    expected = """
        MSH|^~\\&|||||20130715141912||OML^O33^OML_O33|199|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-28R^ROCHE
        SPM|1|1453848101&BARCODE||SERPLAS^^99ROC|||||||P^^HL70369||||||||||||||||SC^^99ROC
        SAC|1||1453848101^BARCODE|||||||50016|3||||||||||||||||||^1^1
        ORC|DC||||||||20130715141912
    """
    assert message == expected


def test_order_from_query_in_non_racks(self):
    message = self.order(
        worklist=self.order_worklist,
        context={
            "query": self.parse(
                """
            MSH|^~\\&|cobas pure||Host||20220826110617+0900||QBP^Q11^QBP_Q11|683|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-27R^ROCHE
            QPD|INIBAR^^99ROC|0|123456|50018|1|||||""|SC^^99ROC|R
            RCP|I|1|R^HL7039
            """
            ),
            "control_id": "199",
        },
    )
    expected = """
        MSH|^~\\&|||||20130715141912||OML^O33^OML_O33|199|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-28R^ROCHE
    PID|1||036666666||^^^^^^U||20110513|F
    SPM|1|123456&BARCODE||SERPLAS^^99ROC|||||||P^^HL70369|||NYME~ANO~036666666~NJF~Prescriber|||||||||||||SC^^99ROC
    SAC|1||123456|||||||50018|1||||||||||||||||||^1^1
    ORC|NW||||||||20130603110342
    TQ1|1||||||||S^^HL70485
    OBR|1|78910||A^^99ROC
    TCD|A^^99ROC|^1^:^10
    ORC|NW||||||||20130603110342
    TQ1|1||||||||S^^HL70485
    OBR|1|78910||B^^99ROC
    TCD|B^^99ROC|^1^:^1
    SPM|2|123456&BARCODE||SERPLAS^^99ROC|||||||P^^HL70369|||NYME~ANO~036666666~NJF~Prescriber|||||||||||||SC^^99ROC
    SAC|1||123456|||||||50018|1||||||||||||||||||^1^1
    ORC|CA||||||||20130603110342
    TQ1|1||||||||S^^HL70485
    OBR|1|78910||C^^99ROC
    TCD|C^^99ROC|^1^:^1
    """
    assert message == expected


def test_order_from_query_in_non_racks_wb_specimen(self):
    message = self.order(
        worklist=self.order_worklist,
        context={
            "query": self.parse(
                """
            MSH|^~\\&|cobas pure||Host||20221125161716+0900||QBP^Q11^QBP_Q11|3180|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-27R^ROCHE
            QPD|INIBAR^^99ROC|138|1718961731|50013|3|||||WB^^HL70487|SC^^99ROC|R
            RCP|I|1|R^HL7039
            """
            ),
            "control_id": "199",
        },
    )
    expected = """
        MSH|^~\\&|||||20130715141912||OML^O33^OML_O33|199|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-28R^ROCHE
        PID|1||036666666||^^^^^^U||20110513|F
        SPM|1|123456&BARCODE||WB^^HL70487|||||||P^^HL70369|||NYME~ANO~036666666~NJF~Prescriber|||||||||||||SC^^99ROC
        SAC|1||123456|||||||50013|3||||||||||||||||||^1^1
        ORC|NW||||||||20130603110342
        TQ1|1||||||||S^^HL70485
        OBR|1|78910||A^^99ROC
        TCD|A^^99ROC|^1^:^10
        ORC|NW||||||||20130603110342
        TQ1|1||||||||S^^HL70485
        OBR|1|78910||B^^99ROC
        TCD|B^^99ROC|^1^:^1
        SPM|2|123456&BARCODE||WB^^HL70487|||||||P^^HL70369|||NYME~ANO~036666666~NJF~Prescriber|||||||||||||SC^^99ROC
        SAC|1||123456|||||||50013|3||||||||||||||||||^1^1
        ORC|CA||||||||20130603110342
        TQ1|1||||||||S^^HL70485
        OBR|1|78910||C^^99ROC
        TCD|C^^99ROC|^1^:^1
    """
    assert message == expected


def test_decode_lab_result(self):
    worklist = self.decode(
        """
        MSH|^~\\&|cobas pure||Host||20220906132520+0900||OUL^R22^OUL_R22|1718|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    PID|||502832||^^^^^^U||19901214|M
    SPM|1|1453811501&BARCODE||SERPLAS^^99ROC|||||||P^^HL70369|||ESSAI~Tox avi 30~502832~ESSAI~|||||
    SAC|||1453811501^BARCODE|||||||50021|1||||||||||||||||||^1^:^1
    OBR|1|14538115||12004^^99ROC|||||||G
    ORC|SC||||CM
    TQ1|||||||||R^^HL70485
    OBX|1|NM|12004^12004^99ROC^^^IHELAW|1|71.9|%^^99ROC||N^^HL70078|||F|||||BRO~REALTIME||e402^ROCHE~2269-07^ROCHE~0^ROCHE|20220906132519||||||||||RSLT
    OBX|2|CE|12004^12004^99ROC^^^IHELAW|1|4^^99ROC|||N^^HL70078|||F|||||BRO~REALTIME||e402^ROCHE~2269-07^ROCHE~0^ROCHE|20220906132519||||||||||RSLT
    OBX|3|EI|QCTID^QC Test ID^99ROC^S_OTHER^Other Supplemental^IHELAW|1|98~92|||N^^HL70078|||F|||||BRO~REALTIME||e402^ROCHE~2269-07^ROCHE~0^ROCHE|20220906132519||||||||||RSLT
    OBX|4|CE|QCSTATE^QC Status^99ROC^S_OTHER^Other Supplemental^IHELAW|1|1^^99ROC|||N^^HL70078|||F|||||BRO~REALTIME||e402^ROCHE~2269-07^ROCHE~0^ROCHE|20220906132519||||||||||RSLT
    OBX|5|NM|11005^11005^99ROC^^^IHELAW|1|*******|IU/mL^^99ROC||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|6|CE|11005^11005^99ROC^^^IHELAW|1|^^99ROC|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    TCD|11005^^99ROC|^1^:^1
    INV|1011005|OK^^HL70383~CURRENT^^99ROC|ASY|741|1|25||||||20221130||||598239
    INV|1018448|OK^^HL70383~CURRENT^^99ROC|PRC|1807|1|1||||||20240430||||654080
    OBX|7|DTM|PT^Pipetting_Time^99ROC^S_OTHER^Other Supplemental^IHELAW|1|20220906125730|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|8|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^Other Supplemental^IHELAW|1|39|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|9|EI|QCTID^QC Test ID^99ROC^S_OTHER^Other Supplemental^IHELAW|1|93~87~86~85|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|10|CE|QCSTATE^QC Status^99ROC^S_OTHER^Other Supplemental^IHELAW|1|1^^99ROC|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|11|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1|0.180 - 650|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|12|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1| - |||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|13|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^Other Supplemental^IHELAW|1| - |||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|14|NM|11005_EFS^11005_EFS^99ROC^S_RAW^Raw Supplemental^IHELAW|1|74270.394|COUNT^^99ROC||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|15|NM|11005_EFV^11005_EFV^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-122.4692|COUNT^^99ROC||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|16|NM|11005_EFC^11005_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|253.2467|COUNT^^99ROC||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|17|NM|11005_PMT^11005_PMT^99ROC^S_RAW^Raw Supplemental^IHELAW|1|16333|COUNT^^99ROC||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132430||||||||||RSLT
    OBX|18|NM|11006^11006^99ROC^^^IHELAW|1|*******|IU/mL^^99ROC||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|19|CE|11006^11006^99ROC^^^IHELAW|1|^^99ROC|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    TCD|11006^^99ROC|^1^:^1
    INV|1011005|OK^^HL70383~CURRENT^^99ROC|ASY|741|1|25||||||20221130||||598239
    INV|1018448|OK^^HL70383~CURRENT^^99ROC|PRC|1807|1|1||||||20240430||||654080
    OBX|20|DTM|PT^Pipetting_Time^99ROC^S_OTHER^Other Supplemental^IHELAW|1|20220906125800|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|21|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^Other Supplemental^IHELAW|1|39|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|22|EI|QCTID^QC Test ID^99ROC^S_OTHER^Other Supplemental^IHELAW|1|94~88~86~85|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|23|CE|QCSTATE^QC Status^99ROC^S_OTHER^Other Supplemental^IHELAW|1|1^^99ROC|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|24|ST|TR_TECHNICALLIMIT^TR_TECHNICALLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1|0.180 - 650|||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|25|ST|TR_REPEATLIMIT^TR_REPEATLIMIT^99ROC^S_OTHER^Other Supplemental^IHELAW|1| - |||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|26|ST|TR_EXPECTEDVALUES^TR_EXPECTEDVALUES^99ROC^S_OTHER^Other Supplemental^IHELAW|1| - |||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|27|NM|11006_EFS^11006_EFS^99ROC^S_RAW^Raw Supplemental^IHELAW|1|356072.45|COUNT^^99ROC||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|28|NM|11006_EFV^11006_EFV^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-122.5015|COUNT^^99ROC||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|29|NM|11006_EFC^11006_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|253.2727|COUNT^^99ROC||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|30|NM|11006_PMT^11006_PMT^99ROC^S_RAW^Raw Supplemental^IHELAW|1|18547|COUNT^^99ROC||SUP^^99ROC|||F|||||BRO~REALTIME||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20220906132500||||||||||RSLT
    OBX|31|NM|12004^12004^99ROC^^^IHELAW|1|""|""||N^^HL70078|||B|||||BRO~REALTIME||e402^ROCHE~2269-07^ROCHE~0^ROCHE|20220906132500||||||||||RSLT
    OBX|32|CE|12004^12004^99ROC^^^IHELAW|1|""|||N^^HL70078|||B|||||BRO~REALTIME||e402^ROCHE~2269-07^ROCHE~0^ROCHE|20220906132500||||||||||RSLT
    INV|1011005|OK^^HL70383~CURRENT^^99ROC|ASY|741|1|25||||||20221130||||598239
    OBX|33|DTM|PT^Pipetting_Time^99ROC^S_OTHER^Other Supplemental^IHELAW|1|20220906125730|||N^^HL70078|||B|||||BRO~REALTIME||e402^ROCHE~2269-07^ROCHE~0^ROCHE|20220906132500||||||||||RSLT
    OBX|34|EI|CombID^Measurement Cell Test Combination ID^99ROC^^^S_OTHER^Other Supplemental^IHELAW|1|1|||N^^HL70078|||B|||||BRO~REALTIME||e402^ROCHE~2269-07^ROCHE~0^ROCHE|20220906132500||||||||||RSLT
    OBX|35|EI|QCTID^QC Test ID^99ROC^S_OTHER^Other Supplemental^IHELAW|1|98~92|||N^^HL70078|||B|||||BRO~REALTIME||e402^ROCHE~2269-07^ROCHE~0^ROCHE|20220906132500||||||||||RSL
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "50021-1",
                "results": [
                    {
                        "code": "12004",
                        "date": datetime.datetime(2022, 9, 6, 13, 25, 19),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "BRO",
                        "unit": "%",
                        "value": "71.9",
                    },
                    {
                        "code": "12004_I",
                        "date": datetime.datetime(2022, 9, 6, 13, 25, 19),
                        "instrument": "e402",
                        "operator": "BRO",
                        "value": "4",
                    },
                    {
                        "code": "11005",
                        "date": datetime.datetime(2022, 9, 6, 13, 24, 30),
                        "flags": ["SUP"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2022, 11, 30, 0, 0),
                                "lot": "598239",
                                "name": "1011005-ASY",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2024, 4, 30, 0, 0),
                                "lot": "654080",
                                "name": "1018448-PRC",
                                "type": "reagent",
                            },
                        ],
                        "operator": "BRO",
                        "unit": "IU/mL",
                        "value": "*******",
                    },
                    {
                        "code": "11005_EFS",
                        "date": datetime.datetime(2022, 9, 6, 13, 24, 30),
                        "flags": ["SUP"],
                        "instrument": "e402",
                        "operator": "BRO",
                        "unit": "COUNT",
                        "value": "74270.394",
                    },
                    {
                        "code": "11005_EFV",
                        "date": datetime.datetime(2022, 9, 6, 13, 24, 30),
                        "flags": ["SUP"],
                        "instrument": "e402",
                        "operator": "BRO",
                        "unit": "COUNT",
                        "value": "-122.4692",
                    },
                    {
                        "code": "11005_EFC",
                        "date": datetime.datetime(2022, 9, 6, 13, 24, 30),
                        "flags": ["SUP"],
                        "instrument": "e402",
                        "operator": "BRO",
                        "unit": "COUNT",
                        "value": "253.2467",
                    },
                    {
                        "code": "11005_PMT",
                        "date": datetime.datetime(2022, 9, 6, 13, 24, 30),
                        "flags": ["SUP"],
                        "instrument": "e402",
                        "operator": "BRO",
                        "unit": "COUNT",
                        "value": "16333",
                    },
                    {
                        "code": "11006",
                        "date": datetime.datetime(2022, 9, 6, 13, 25),
                        "flags": ["SUP"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2022, 11, 30, 0, 0),
                                "lot": "598239",
                                "name": "1011005-ASY",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2024, 4, 30, 0, 0),
                                "lot": "654080",
                                "name": "1018448-PRC",
                                "type": "reagent",
                            },
                        ],
                        "operator": "BRO",
                        "unit": "IU/mL",
                        "value": "*******",
                    },
                    {
                        "code": "11006_EFS",
                        "date": datetime.datetime(2022, 9, 6, 13, 25),
                        "flags": ["SUP"],
                        "instrument": "e402",
                        "operator": "BRO",
                        "unit": "COUNT",
                        "value": "356072.45",
                    },
                    {
                        "code": "11006_EFV",
                        "date": datetime.datetime(2022, 9, 6, 13, 25),
                        "flags": ["SUP"],
                        "instrument": "e402",
                        "operator": "BRO",
                        "unit": "COUNT",
                        "value": "-122.5015",
                    },
                    {
                        "code": "11006_EFC",
                        "date": datetime.datetime(2022, 9, 6, 13, 25),
                        "flags": ["SUP"],
                        "instrument": "e402",
                        "operator": "BRO",
                        "unit": "COUNT",
                        "value": "253.2727",
                    },
                    {
                        "code": "11006_PMT",
                        "date": datetime.datetime(2022, 9, 6, 13, 25),
                        "flags": ["SUP"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2022, 11, 30, 0, 0),
                                "lot": "598239",
                                "name": "1011005-ASY",
                                "type": "reagent",
                            }
                        ],
                        "operator": "BRO",
                        "unit": "COUNT",
                        "value": "18547",
                    },
                ],
                "sample_id": "1453811501",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_inv_after_sac(self):
    worklist = self.decode(
        """
        MSH|^~\\&|cobas pure||Host||20230113104545+0900||OUL^R22^OUL_R22|5|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|672&CONTROL||SERPLAS^^99ROC|||||||Q^^HL70369|||~~~~|||||20230331||||||||
    SAC|||672^CONTROL|||||||637079|
    INV|672^PC ACOV2S2^99ROC|OK^^HL703843|CO^^HL70384|^^99ROC
    OBR|1|""||10230^^99ROC|||||||
    ORC|SC||||CM
    TQ1|||||||||R^^HL70485
    OBX|1|NM|10230^10230^99ROC^^^IHELAW|1|7.24|U/mL^^99ROC||N^^HL70078|||F|||||bonnetn~BATCH||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20230113100148||5335||||||||RSLT
    OBX|2|CE|10230^10230^99ROC^^^IHELAW|1|^^99ROC|||N^^HL70078|||F|||||bonnetn~BATCH||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20230113100148||5335||||||||RSLT
    TCD|10230^^99ROC|^1^:^1
    INV|1310230|OK^^HL70383~CURRENT^^99ROC|ASY|10781|1|16||||||20230531||||647392
    INV|1018448|OK^^HL70383~CURRENT^^99ROC|PRC|1640|1|1||||||20240831||||689579
    OBX|3|DTM|PT^Pipetting_Time^99ROC^S_OTHER^Other Supplemental^IHELAW|1|20230113094318|||N^^HL70078|||F|||||bonnetn~BATCH||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20230113100148||5335||||||||RSLT
    OBX|4|EI|CalibrationID^CalibrationID^99ROC^S_OTHER^Other Supplemental^IHELAW|1|341|||N^^HL70078|||F|||||bonnetn~BATCH||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20230113100148||5335||||||||RSLT
    OBX|5|NM|QC_TARGET^QC_TARGET^99ROC^S_OTHER^Other Supplemental^IHELAW|1|8.61|U/mL^^99ROC||N^^HL70078|||F|||||bonnetn~BATCH||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20230113100148||5335||||||||RSLT
    OBX|6|NM|QC_SD_RANGE^QC_SD_RANGE^99ROC^S_OTHER^Other Supplemental^IHELAW|1|0.861|U/mL^^99ROC||N^^HL70078|||F|||||bonnetn~BATCH||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20230113100148||5335||||||||RSLT
    OBX|7|NM|10230_EFS^10230_EFS^99ROC^S_RAW^Raw Supplemental^IHELAW|1|19864.459|COUNT^^99ROC||N^^HL70078|||F|||||bonnetn~BATCH||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20230113100148||||||||||RSLT
    OBX|8|NM|10230_EFV^10230_EFV^99ROC^S_RAW^Raw Supplemental^IHELAW|1|-116.4648|COUNT^^99ROC||N^^HL70078|||F|||||bonnetn~BATCH||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20230113100148||||||||||RSLT
    OBX|9|NM|10230_EFC^10230_EFC^99ROC^S_RAW^Raw Supplemental^IHELAW|1|243.4462|COUNT^^99ROC||N^^HL70078|||F|||||bonnetn~BATCH||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20230113100148||||||||||RSLT
    OBX|10|NM|10230_PMT^10230_PMT^99ROC^S_RAW^Raw Supplemental^IHELAW|1|15832|COUNT^^99ROC||N^^HL70078|||F|||||bonnetn~BATCH||e402^ROCHE~2256-06^ROCHE~1^ROCHE|20230113100148||||||||||RSL
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "637079",
                "results": [
                    {
                        "code": "10230",
                        "date": datetime.datetime(2023, 1, 13, 10, 1, 48),
                        "flags": ["N"],
                        "instrument": "e402",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2023, 5, 31, 0, 0),
                                "lot": "647392",
                                "name": "1310230-ASY",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2024, 8, 31, 0, 0),
                                "lot": "689579",
                                "name": "1018448-PRC",
                                "type": "reagent",
                            },
                        ],
                        "operator": "bonnetn",
                        "unit": "U/mL",
                        "value": "7.24",
                    },
                    {
                        "code": "QC_TARGET",
                        "date": datetime.datetime(2023, 1, 13, 10, 1, 48),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "bonnetn",
                        "unit": "U/mL",
                        "value": "8.61",
                    },
                    {
                        "code": "QC_SD_RANGE",
                        "date": datetime.datetime(2023, 1, 13, 10, 1, 48),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "bonnetn",
                        "unit": "U/mL",
                        "value": "0.861",
                    },
                    {
                        "code": "10230_EFS",
                        "date": datetime.datetime(2023, 1, 13, 10, 1, 48),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "bonnetn",
                        "unit": "COUNT",
                        "value": "19864.459",
                    },
                    {
                        "code": "10230_EFV",
                        "date": datetime.datetime(2023, 1, 13, 10, 1, 48),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "bonnetn",
                        "unit": "COUNT",
                        "value": "-116.4648",
                    },
                    {
                        "code": "10230_EFC",
                        "date": datetime.datetime(2023, 1, 13, 10, 1, 48),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "bonnetn",
                        "unit": "COUNT",
                        "value": "243.4462",
                    },
                    {
                        "code": "10230_PMT",
                        "date": datetime.datetime(2023, 1, 13, 10, 1, 48),
                        "flags": ["N"],
                        "instrument": "e402",
                        "operator": "bonnetn",
                        "unit": "COUNT",
                        "value": "15832",
                    },
                ],
                "sample_id": "637079",
                "type": "control",
            }
        ],
    }
    assert worklist == expected
