# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "dxi9000"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&
    P|1|036666666|L036666666||NYME^ANO||20110513|F|||||ATTENDING^LAGRANGE^Joseph-Louis||||||||||||LOC
    C|1|L|^Commentaire patient no1|G
    C|2|L|^Commentaire patient no2|G
    O|1|123456||^^^A|R||20130513114242||||N||||Serum|PRESCRIBER^Prescriber|||||||||O
    C|1|L|^test retour chariot|G
    O|2|123456||^^^B|R||20130513114242||||A||||Serum|PRESCRIBER^Prescriber|||||||||O
    C|1|L|^test retour chariot|G
    O|3|123456||^^^C|R||20130513114242||||C||||Serum|PRESCRIBER^Prescriber|||||||||O
    C|1|L|^test retour chariot|G
    L|1|N
    """
    assert message == expected


def test_encode_void_order(self):
    message = self.encode(
        self.void_worklist,
        context={
            "query": self.parse(
                """
            H|\\^&
            Q|1|^SAMPLE001||||||||||O
            L|1|N
            """
            )
        },
    )
    expected = """
        H|\\^&
        P|1
        O|1|SAMPLE001|||||||||||||||||||||||Y
        L|1|N
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&
    Q|1|^SAMPLE001||||||||||O
    L|1|N
    """
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "SAMPLE001", "type": "patient"}],
    }
    assert worklist == expected


def test_decode_results(self):
    worklist = self.decode(
        """
    H|\\^&|||DxI 9000^A12345^v1.0||||||||LIS2-A2|20180823200509
    P|1||PID101||Doe^John^T||20180823|U|||||^Frankenstein^Victor^V^^Dr.||||||||||||ER
    C|1|P|^Patient exhibited signs of a very Severe illness|G
    O|1|S12345||^^^13B|R||||||||||Serum||||||||||F
    R|1|^^^13B|>123^>123^Negative|||||F||||20180823200508|3482349873249279^DxI 9000^Beckman Coulter
    C|1|I|HH^Very High|I
    M|1|119|DIL1^Beckman Diluent|OK^OK|DI^Diluent|121121|RE397200777
    M|2|119|R1^Beckman Reagent R1|OK^OK|SR^Single Test Reagent|7563152|LOT99996342
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "13B",
                        "comments": ["Very High"],
                        "date": datetime.datetime(2018, 8, 23, 20, 5, 8),
                        "flags": ["HH"],
                        "instrument": "DxI 9000 3482349873249279",
                        "lots": [
                            {
                                "lot": "RE397200777",
                                "name": "Beckman Diluent",
                                "type": "reagent",
                            },
                            {
                                "lot": "LOT99996342",
                                "name": "Beckman Reagent R1",
                                "type": "reagent",
                            },
                        ],
                        "unit": "",
                        "value": ">123",
                    },
                    {
                        "code": "Int_13B",
                        "comments": ["Very High"],
                        "date": datetime.datetime(2018, 8, 23, 20, 5, 8),
                        "flags": ["HH"],
                        "instrument": "DxI 9000 3482349873249279",
                        "lots": [
                            {
                                "lot": "RE397200777",
                                "name": "Beckman Diluent",
                                "type": "reagent",
                            },
                            {
                                "lot": "LOT99996342",
                                "name": "Beckman Reagent R1",
                                "type": "reagent",
                            },
                        ],
                        "value": "Negative",
                    },
                ],
                "sample_id": "S12345",
                "specimen": "Serum",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    H|\\^&|||DxI 9000^300032^1.16.0.140||||||||LIS2-A2|20220624180025
    P|1|Lot76345267
    O|1|BioRad-Level2||14J^A1c2^99BCI^14J|R||||||Q||||||||||||||F
    R|1|14J^A1c2^99BCI^14J|6.968^6.968|mmol/L||||F||||20220625014138|300032^DxI 9000^Beckman Coulter
    O|2|BioRad-Level2||AFP^AFP^99BCI^AFP|R||||||Q||||||||||||||F
    R|1|AFP^AFP^99BCI^AFP|22.829^22.829|mmol/L||||F||||20220625012910|300032^DxI 9000^Beckman Coulter
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "BioRad-Level2",
                "results": [
                    {
                        "code": "14J",
                        "comments": [],
                        "date": datetime.datetime(2022, 6, 25, 1, 41, 38),
                        "flags": [],
                        "instrument": "DxI 9000 300032",
                        "lots": [],
                        "unit": "mmol/L",
                        "value": "6.968",
                    },
                    {
                        "code": "AFP",
                        "comments": [],
                        "date": datetime.datetime(2022, 6, 25, 1, 29, 10),
                        "flags": [],
                        "instrument": "DxI 9000 300032",
                        "lots": [],
                        "unit": "mmol/L",
                        "value": "22.829",
                    },
                ],
                "sample_id": "BioRad-Level2",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_empty_results(self):
    worklist = self.decode(
        """
    H|\\^&
    P|1||PID001
    O|1|S1000||HIVCombo^HIVCombo^99BCI^HIVCombo|R||||||||||Serum||||||||||X
    R|1|HIVCombo^HIVCombo^99BCI^HIVCombo|No result^^No result|||||X||||20221001195108|3482349873249279^DxI 9000^Beckman Coulter
    C|1|I|NRT^No result, timeout|I
    R|2|Ag^Ag^99BCI^Ag|No result^^No result|||||X||||20221001195108|MP120^DxI 9000^Beckman Coulter
    C|1|I|NRT^No result, timeout|I
    R|3|Ab^Ab^99BCI^Ab|No result^^No result|||||X||||20221001195108|MP120^DxC^beckman
    C|1|I|NRT^No result, timeout|I
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "HIVCombo",
                        "comments": ["No result, timeout"],
                        "date": datetime.datetime(2022, 10, 1, 19, 51, 8),
                        "flags": ["NRT"],
                        "instrument": "DxI 9000 3482349873249279",
                        "lots": [],
                        "unit": "",
                        "value": "No result",
                    },
                    {
                        "code": "Int_HIVCombo",
                        "comments": ["No result, timeout"],
                        "date": datetime.datetime(2022, 10, 1, 19, 51, 8),
                        "flags": ["NRT"],
                        "instrument": "DxI 9000 3482349873249279",
                        "lots": [],
                        "value": "No result",
                    },
                    {
                        "code": "Ag",
                        "comments": ["No result, timeout"],
                        "date": datetime.datetime(2022, 10, 1, 19, 51, 8),
                        "flags": ["NRT"],
                        "instrument": "DxI 9000 MP120",
                        "lots": [],
                        "unit": "",
                        "value": "No result",
                    },
                    {
                        "code": "Int_Ag",
                        "comments": ["No result, timeout"],
                        "date": datetime.datetime(2022, 10, 1, 19, 51, 8),
                        "flags": ["NRT"],
                        "instrument": "DxI 9000 MP120",
                        "lots": [],
                        "value": "No result",
                    },
                    {
                        "code": "Ab",
                        "comments": ["No result, timeout"],
                        "date": datetime.datetime(2022, 10, 1, 19, 51, 8),
                        "flags": ["NRT"],
                        "instrument": "DxC MP120",
                        "lots": [],
                        "unit": "",
                        "value": "No result",
                    },
                    {
                        "code": "Int_Ab",
                        "comments": ["No result, timeout"],
                        "date": datetime.datetime(2022, 10, 1, 19, 51, 8),
                        "flags": ["NRT"],
                        "instrument": "DxC MP120",
                        "lots": [],
                        "value": "No result",
                    },
                ],
                "sample_id": "S1000",
                "specimen": "Serum",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
