# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist

parser_class = "eurobio_i2p"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||||||||||20130715141912
    P|1|036666666|123456||NYME^ANO||20110513|F|||||PRESCRIBER
    O|1|123456||^^^A|||20130513114242||||||||Serum||||||||||X
    O|2|123456||^^^B|||20130513114242||||||||Serum||||||||||X
    L|1|N"""
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||
    Q|1|40095810||ALL||||||||O
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "40095810", "type": "patient"}],
    }
    assert worklist == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
        H|\\^&|||
        Q|1|40095810||ALL||||||||O
        L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&||||||||||||20130715141912
    L|1|N
    """
    assert result == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&|||
    P|1||880041650001
    O|1|||^^^2COVIGE||20200416162112
    R|1|^^^2COVIGE|2.96|Ratio|
    P|2||880041650001
    O|1|||^^^2COVIAE||20200416162112
    R|1|^^^2COVIAE|3.14|Ratio|
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "2COVIGE",
                        "date": None,
                        "flags": [],
                        "lots": [],
                        "value": "2.96",
                    },
                    {
                        "code": "2COVIAE",
                        "date": None,
                        "flags": [],
                        "lots": [],
                        "value": "3.14",
                    },
                ],
                "sample_id": "880041650001",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
