# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "zenitupxml"


def test_void_order(self):
    _order_message = """
    <GROUP NAME="WORKLIST" TYPE="HOST">
        <GROUP TYPE="Sample">
            <GROUP ID="123456" TYPE="Patient">
                <PARAM ID="Name" TYPE="NYME ANO"/>
                <PARAM ID="BirthDate" TYPE="20110513"/>
                <PARAM ID="Sex" TYPE="F"/>
            </GROUP>
            <GROUP ID="" TYPE="Assays">
                <GROUP ID="A" STATE="Host" TYPE="Assay">
                    <GROUP TYPE="Dilutions">
                        <PARAM ID="1:10" TYPE="Dilution"/>
                    </GROUP>
                </GROUP>
                <GROUP ID="B" STATE="Host" TYPE="Assay"/>
             </GROUP>
        </GROUP>
    </GROUP>"""
    message = self.order()
    assert message == _order_message


def test_decode_result(self):
    date_result = datetime.datetime(2020, 3, 20, 11, 2, 40)
    worklist = self.decode(
        """
    <GROUP ID="HostOut">
        <GROUP ID="SoftwareInfo">
            <PARAM TYPE="String" ID="Name">Zenit UP</PARAM>
            <PARAM TYPE="String" ID="Version">2.12.51</PARAM>
        </GROUP>
        <GROUP ID="SessionInfo" FILTER="txxxr">
            <PARAM TYPE="String" ID="ID" FILTER="-wwwr">--- PHOSPHO 20.03.2020</PARAM>
            <PARAM TYPE="String" ID="User" FILTER="-rrrr">TUZAL</PARAM>
            <PARAM TYPE="DateTime" ID="LastChangeDateTime" FILTER="-rrrr">20/03/2020 11:02:40</PARAM>
            <PARAM TYPE="DateTime" ID="Execution" FILTER="-rrrr">20/03/2020 11:02:40</PARAM>
            <PARAM TYPE="String" ID="PhaseSelection" FILTER="-----">FullSession</PARAM>
            <PARAM TYPE="String" ID="DoctorHeader"/>
            <PARAM TYPE="String" ID="LaboratoryHeader"/>
        </GROUP>
        <GROUP TYPE="Instrument" ID="APEZenit.228">
            <GROUP ID="Standards">
                <GROUP TYPE="Assay" ID="B2GP1 EUROIMMUN">
                    <PARAM TYPE="CurveType" ID="Curve">PointToPoint</PARAM>
                    <PARAM TYPE="String" ID="CurveUsage">New</PARAM>
                    <GROUP TYPE="Standard" ID="1">
                        <PARAM TYPE="Double" ID="Value">200</PARAM>
                        <GROUP TYPE="Position" ID="2">
                            <PARAM ID="OD">1.455</PARAM>
                        </GROUP>
                    </GROUP>
                    <GROUP TYPE="Standard" ID="2">
                        <PARAM TYPE="Double" ID="Value">20</PARAM>
                        <GROUP TYPE="Position" ID="3">
                            <PARAM ID="OD">0.266</PARAM>
                        </GROUP>
                    </GROUP>
                    <GROUP TYPE="Standard" ID="3">
                        <PARAM TYPE="Double" ID="Value">2</PARAM>
                        <GROUP TYPE="Position" ID="4">
                            <PARAM ID="OD">0.019</PARAM>
                        </GROUP>
                    </GROUP>
                </GROUP>
                <GROUP TYPE="Assay" ID="CARDIO NEW">
                    <PARAM TYPE="CurveType" ID="Curve">PointToPoint</PARAM>
                    <PARAM TYPE="String" ID="CurveUsage">New</PARAM>
                    <GROUP TYPE="Standard" ID="1">
                        <PARAM TYPE="Double" ID="Value">120</PARAM>
                        <GROUP TYPE="Position" ID="98">
                            <PARAM ID="OD">1.600</PARAM>
                        </GROUP>
                    </GROUP>
                    <GROUP TYPE="Standard" ID="2">
                        <PARAM TYPE="Double" ID="Value">12</PARAM>
                        <GROUP TYPE="Position" ID="99">
                            <PARAM ID="OD">0.354</PARAM>
                        </GROUP>
                    </GROUP>
                    <GROUP TYPE="Standard" ID="3">
                        <PARAM TYPE="Double" ID="Value">2</PARAM>
                        <GROUP TYPE="Position" ID="100">
                            <PARAM ID="OD">0.082</PARAM>
                        </GROUP>
                    </GROUP>
                </GROUP>
            </GROUP>
        </GROUP>
    </GROUP>
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "Position 98",
                "results": [
                    {
                        "code": "B2GP1 EUROIMMUN",
                        "instrument": "APEZenit.228",
                        "value": "200",
                        "date": date_result,
                    },
                    {
                        "code": "CARDIO NEW",
                        "instrument": "APEZenit.228",
                        "value": "120",
                        "date": date_result,
                    },
                ],
                "sample_id": "1",
                "type": "patient",
            },
            {
                "rack": "Position 99",
                "results": [
                    {
                        "code": "B2GP1 EUROIMMUN",
                        "instrument": "APEZenit.228",
                        "value": "20",
                        "date": date_result,
                    },
                    {
                        "code": "CARDIO NEW",
                        "instrument": "APEZenit.228",
                        "value": "12",
                        "date": date_result,
                    },
                ],
                "sample_id": "2",
                "type": "patient",
            },
            {
                "rack": "Position 100",
                "results": [
                    {
                        "code": "B2GP1 EUROIMMUN",
                        "instrument": "APEZenit.228",
                        "value": "2",
                        "date": date_result,
                    },
                    {
                        "code": "CARDIO NEW",
                        "instrument": "APEZenit.228",
                        "value": "2",
                        "date": date_result,
                    },
                ],
                "sample_id": "3",
                "type": "patient",
            },
        ],
    }
    assert worklist == expected
